"""Bootstrap for loading custom modules into PyBOINC

Known issues:
 - Since site.py is never imported, there are some differences
   from the standard Python release 
"""

import sys, boinc

LIB_NAME = ""

def bootstrap_path():
    """Initializes sys.path based on the platform and given zip library
    """
    
    global LIB_NAME
    
    app_name = boinc.get_app_name().lower()
    supported_platforms = ["linux", "windows", "apple"]
    
    for plat in supported_platforms:
        if plat in app_name:
            app_name = plat
            break
    
    if app_name not in supported_platforms:
        print "Error: unsupported platform"
        print "The platform could not be determined from the app name"
        print "linux, windows or apple is expected to be part of the app name"
        sys.exit(1)
    
    platmod = None
    platmods = [plat for plat in sys.path if "plat-" in plat]
    if len(platmods):
        platmod = platmods[0]
        platmod = platmod[platmod.find("plat-"):]
    
    LIB_NAME =  app_name + "_" + sys.argv[1]

    sys.path.insert(1, sys.path[0] + "/DLLs")
    sys.path.insert(1, LIB_NAME + "/lib-tk")

    if platmod:
        sys.path.insert(1, LIB_NAME + "/" + platmod)
        
    sys.path.insert(1, LIB_NAME)

# first setup path before importing anything from the standard library
bootstrap_path()

# now pull in the other modules required to run the application
import os, zipfile, re, traceback, platform
import __builtin__

# cleanup relative paths incase someone changes the cwd
for i in range(len(sys.path)):
    sys.path[i] = os.path.abspath(sys.path[i])

def main():
    """Main method responsible for bootstrapping PyBOINC
    """
    
    global LIB_NAME
    
    # we must receive at least one argument
    # indicating which module to load
    if len(sys.argv) == 0:
        print "Error: no module specified!"
        sys.exit(1)
    
    # Extracting dynamic libraries into folders
    # so that the OS can load them. These paths
    # must be added to a env var that that is OS specific
    sep = os.path.pathsep
    if platform.system() == "Windows":
        env = "PATH"
    else:
        env = "LD_LIBRARY_PATH"

    path = os.environ.get(env,"")
    
    lib_base = os.path.abspath("." + os.path.sep + "libs" + os.path.sep)
    #for lib in [sys.argv[1]]:
    #if not lib.lower().endswith(".zip"): break
    print "lib_base", lib_base
    main_lib = os.path.join(lib_base, os.path.basename(LIB_NAME) + os.path.sep)
    print "main_lib", main_lib 
    if not os.path.exists(main_lib):
        os.makedirs(main_lib)
    extract(LIB_NAME, main_lib, "dynamic-libs/(.*)")
    sys.path += [os.path.join(main_lib, "dynamic-libs" + os.path.sep)]
    path += sep + os.path.join(main_lib, "dynamic-libs" + os.path.sep)
    del sys.argv[1]
    
    os.putenv(env, path)
    
    sethelper()
    setquit()
    
    sys.path[0] = os.path.abspath(os.path.dirname(sys.argv[0]))
    
    # import the module and store a local reference
    try:
        # execution now takes place in C++ so that
        # the modules sees __name__ as "__main__"
        
        #execfile(sys.argv[0], {})
        pass
    except Exception:
        print "Exception:"
        print '-'*60
        traceback.print_exc(file=sys.stderr)
        print '-'*60
    
def extract(zip_obj, root, pattern):
    """Extracts all files matching a given regex pattern
    All missing directories will be created as needed to create
    the given file.
    """
    if not os.path.exists(root):
        os.mkdir(root)
    if isinstance(zip_obj, str):
        zip_obj = zipfile.ZipFile(zip_obj, "r")
    for name in zip_obj.namelist():
        if not re.match(pattern, name):
            continue
        if name.endswith("/") or name.endswith("\\"):
            continue
        filename = os.path.join(root, name)
        filepath = os.path.dirname(filename) + os.path.sep

        print os.path.join(root, name)

        if filepath == filename:
            continue
        if not os.path.exists(filepath):
            print filepath
            os.makedirs(filepath)

        curfile = open(os.path.join(root, name), "wb")
        try:
            curfile.write(zip_obj.read(name))
        finally:
            curfile.close()
    zip_obj.close()
    print "exited startup.py"

# ---------------------------------
# taken from Python 2.6 site.py
# ---------------------------------
class _Helper(object):
    """Define the built-in 'help'.
    This is a wrapper around pydoc.help (with a twist).

    """

    def __repr__(self):
        return "Type help() for interactive help, " \
               "or help(object) for help about object."
    def __call__(self, *args, **kwds):
        import pydoc
        return pydoc.help(*args, **kwds)

def sethelper():
    __builtin__.help = _Helper()

def setquit():
    """Define new built-ins 'quit' and 'exit'.
    These are simply strings that display a hint on how to exit.

    """
    if os.sep == ':':
        eof = 'Cmd-Q'
    elif os.sep == '\\':
        eof = 'Ctrl-Z plus Return'
    else:
        eof = 'Ctrl-D (i.e. EOF)'

    class Quitter(object):
        def __init__(self, name):
            self.name = name
        def __repr__(self):
            return 'Use %s() or %s to exit' % (self.name, eof)
        def __call__(self, code=None):
            # Shells like IDLE catch the SystemExit, but listen when their
            # stdin wrapper is closed.
            try:
                sys.stdin.close()
            except:
                pass
            raise SystemExit(code)
    __builtin__.quit = Quitter('quit')
    __builtin__.exit = Quitter('exit')
# ---------------------------------

main()
